@@ -1,27 +1,27 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2003-2005 Edgewall Software
-# Copyright (C) 2003-2005 Jonas Borgstrm <jonas@edgewall.com>
+# Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
-# Author: Jonas Borgstrm <jonas@edgewall.com>
-
-from __future__ import generators
+# Author: Jonas Borgström <jonas@edgewall.com>
 
 import os
 
-from trac import db, db_default, util
-from trac.config import Configuration
+from trac import db_default, util
+from trac.config import *
 from trac.core import Component, ComponentManager, implements, Interface, \
                       ExtensionPoint, TracError
+from trac.db import DatabaseManager
+from trac.versioncontrol import RepositoryManager
 
 __all__ = ['Environment', 'IEnvironmentSetupParticipant', 'open_environment']
 
@@ -62,29 +62,69 @@ class Environment(Component, ComponentManager):
     """   
     setup_participants = ExtensionPoint(IEnvironmentSetupParticipant)
 
-    def __init__(self, path, create=False, db_str=None):
+    base_url = Option('trac', 'base_url', '',
+        """Base URL of the Trac deployment.
+        
+        In most configurations, Trac will automatically reconstruct the URL
+        that is used to access it automatically. However, in more complex
+        setups, usually involving running Trac behind a HTTP proxy, you may
+        need to use this option to force Trac to use the correct URL.""")
+
+    project_name = Option('project', 'name', 'My Project',
+        """Name of the project.""")
+
+    project_description = Option('project', 'descr', 'My example project',
+        """Short description of the project.""")
+
+    project_url = Option('project', 'url', 'http://example.org/',
+        """URL of the main project web site.""")
+
+    project_footer = Option('project', 'footer',
+                            'Visit the Trac open source project at<br />'
+                            '<a href="http://trac.edgewall.org/">'
+                            'http://trac.edgewall.org/</a>',
+        """Page footer text (right-aligned).""")
+
+    project_icon = Option('project', 'icon', 'common/trac.ico',
+        """URL of the icon of the project.""")
+
+    log_type = Option('logging', 'log_type', 'none',
+        """Logging facility to use.
+        
+        Should be one of (`none`, `file`, `stderr`, `syslog`, `winlog`).""")
+
+    log_file = Option('logging', 'log_file', 'trac.log',
+        """If `log_type` is `file`, this should be a path to the log-file.""")
+
+    log_level = Option('logging', 'log_level', 'DEBUG',
+        """Level of verbosity in log.
+        
+        Should be one of (`CRITICAL`, `ERROR`, `WARN`, `INFO`, `DEBUG`).""")
+
+    def __init__(self, path, create=False, options=[]):
         """Initialize the Trac environment.
         
         @param path:   the absolute path to the Trac environment
         @param create: if `True`, the environment is created and populated with
                        default data; otherwise, the environment is expected to
                        already exist.
-        @param db_str: the database connection string
+        @param options: A list of `(section, name, value)` tuples that define
+                        configuration options
         """
         ComponentManager.__init__(self)
 
         self.path = path
-        self.__cnx_pool = None
-        if create:
-            self.create(db_str)
-        else:
-            self.verify()
-            self.load_config()
+        self.setup_config(load_defaults=create)
         self.setup_log()
 
         from trac.loader import load_components
         load_components(self)
 
+        if create:
+            self.create(options)
+        else:
+            self.verify()
+
         if create:
             for setup_participant in self.setup_participants:
                 setup_participant.environment_created()
@@ -106,7 +146,7 @@ class Environment(Component, ComponentManager):
         This is called by the `ComponentManager` base class when a component is
         about to be activated. If this method returns false, the component does
         not get activated."""
-        if not isinstance(cls, (str, unicode)):
+        if not isinstance(cls, basestring):
             component_name = (cls.__module__ + '.' + cls.__name__).lower()
         else:
             component_name = cls.lower()
@@ -120,6 +160,11 @@ class Environment(Component, ComponentManager):
                     and component_name.startswith(pattern[:-1]):
                 return enabled
 
+        # versioncontrol components are enabled if the repository is configured
+        # FIXME: this shouldn't be hardcoded like this
+        if component_name.startswith('trac.versioncontrol.'):
+            return self.config.get('trac', 'repository_dir') != ''
+
         # By default, all components in the trac package are enabled
         return component_name.startswith('trac.')
 
@@ -127,42 +172,29 @@ class Environment(Component, ComponentManager):
         """Verify that the provided path points to a valid Trac environment
         directory."""
         fd = open(os.path.join(self.path, 'VERSION'), 'r')
-        assert fd.read(26) == 'Trac Environment Version 1'
-        fd.close()
+        try:
+            assert fd.read(26) == 'Trac Environment Version 1'
+        finally:
+            fd.close()
 
     def get_db_cnx(self):
         """Return a database connection from the connection pool."""
-        if not self.__cnx_pool:
-            self.__cnx_pool = db.get_cnx_pool(self)
-        return self.__cnx_pool.get_cnx()
+        return DatabaseManager(self).get_connection()
 
-    def shutdown(self):
+    def shutdown(self, tid=None):
         """Close the environment."""
-        if self.__cnx_pool:
-            self.__cnx_pool.shutdown()
-            self.__cnx_pool = None
+        RepositoryManager(self).shutdown(tid)
+        DatabaseManager(self).shutdown(tid)
 
     def get_repository(self, authname=None):
         """Return the version control repository configured for this
         environment.
         
-        The repository is wrapped in a `CachedRepository`.
-        
         @param authname: user name for authorization
         """
-        from trac.versioncontrol.cache import CachedRepository
-        from trac.versioncontrol.svn_authz import SubversionAuthorizer
-        from trac.versioncontrol.svn_fs import SubversionRepository
-        repos_dir = self.config.get('trac', 'repository_dir')
-        if not repos_dir:
-            raise EnvironmentError, 'Path to repository not configured'
-        authz = None
-        if authname:
-            authz = SubversionAuthorizer(self, authname)
-        repos = SubversionRepository(repos_dir, authz, self.log)
-        return CachedRepository(self.get_db_cnx(), repos, authz, self.log)
+        return RepositoryManager(self).get_repository(authname)
 
-    def create(self, db_str=None):
+    def create(self, options=[]):
         """Create the basic directory structure of the environment, initialize
         the database and populate the configuration file with default values."""
         def _create_file(fname, data=None):
@@ -171,7 +203,8 @@ class Environment(Component, ComponentManager):
             fd.close()
 
         # Create the directory structure
-        os.mkdir(self.path)
+        if not os.path.exists(self.path):
+            os.mkdir(self.path)
         os.mkdir(self.get_log_dir())
         os.mkdir(self.get_htdocs_dir())
         os.mkdir(os.path.join(self.path, 'plugins'))
@@ -182,19 +215,18 @@ class Environment(Component, ComponentManager):
                      'Trac Environment Version 1\n')
         _create_file(os.path.join(self.path, 'README'),
                      'This directory contains a Trac environment.\n'
-                     'Visit http://trac.edgewall.com/ for more information.\n')
+                     'Visit http://trac.edgewall.org/ for more information.\n')
 
         # Setup the default configuration
         os.mkdir(os.path.join(self.path, 'conf'))
         _create_file(os.path.join(self.path, 'conf', 'trac.ini'))
-        self.load_config()
-        for section, name, value in db_default.default_config:
+        self.setup_config(load_defaults=True)
+        for section, name, value in options:
             self.config.set(section, name, value)
-        self.config.set('trac', 'database', db_str)
         self.config.save()
 
         # Create the database
-        db.init_db(self.path, db_str)
+        DatabaseManager(self).init_db()
 
     def get_version(self, db=None):
         """Return the current version of the database."""
@@ -205,11 +237,15 @@ class Environment(Component, ComponentManager):
         row = cursor.fetchone()
         return row and int(row[0])
 
-    def load_config(self):
+    def setup_config(self, load_defaults=False):
         """Load the configuration file."""
         self.config = Configuration(os.path.join(self.path, 'conf', 'trac.ini'))
-        for section, name, value in db_default.default_config:
-            self.config.setdefault(section, name, value)
+        if load_defaults:
+            for section, default_options in self.config.defaults().iteritems():
+                for name, value in default_options.iteritems():
+                    if self.config.has_site_option(section, name):
+                        value = None
+                    self.config.set(section, name, value)
 
     def get_templates_dir(self):
         """Return absolute path to the templates directory."""
@@ -226,13 +262,11 @@ class Environment(Component, ComponentManager):
     def setup_log(self):
         """Initialize the logging sub-system."""
         from trac.log import logger_factory
-        logtype = self.config.get('logging', 'log_type')
-        loglevel = self.config.get('logging', 'log_level')
-        logfile = self.config.get('logging', 'log_file')
-        if not os.path.isabs(logfile):
+        logtype = self.log_type
+        logfile = self.log_file
+        if logtype == 'file' and not os.path.isabs(logfile):
             logfile = os.path.join(self.get_log_dir(), logfile)
-        logid = self.path # Env-path provides process-unique ID
-        self.log = logger_factory(logtype, logfile, loglevel, logid)
+        self.log = logger_factory(logtype, logfile, self.log_level, self.path)
 
     def get_known_users(self, cnx=None):
         """Generator that yields information about all known users, i.e. users
@@ -248,12 +282,12 @@ class Environment(Component, ComponentManager):
         if not cnx:
             cnx = self.get_db_cnx()
         cursor = cnx.cursor()
-        cursor.execute("SELECT DISTINCT s.sid, n.var_value, e.var_value "
+        cursor.execute("SELECT DISTINCT s.sid, n.value, e.value "
                        "FROM session AS s "
-                       " LEFT JOIN session AS n ON (n.sid=s.sid "
-                       "  AND n.authenticated=1 AND n.var_name = 'name') "
-                       " LEFT JOIN session AS e ON (e.sid=s.sid "
-                       "  AND e.authenticated=1 AND e.var_name = 'email') "
+                       " LEFT JOIN session_attribute AS n ON (n.sid=s.sid "
+                       "  and n.authenticated=1 AND n.name = 'name') "
+                       " LEFT JOIN session_attribute AS e ON (e.sid=s.sid "
+                       "  AND e.authenticated=1 AND e.name = 'email') "
                        "WHERE s.authenticated=1 ORDER BY s.sid")
         for username,name,email in cursor:
             yield username, name, email
@@ -268,7 +302,7 @@ class Environment(Component, ComponentManager):
 
         db_str = self.config.get('trac', 'database')
         if not db_str.startswith('sqlite:'):
-            raise EnvironmentError, 'Can only backup sqlite databases'
+            raise EnvironmentError('Can only backup sqlite databases')
         db_name = os.path.join(self.path, db_str[7:])
         if not dest:
             dest = '%s.%i.bak' % (db_name, self.get_version())
@@ -324,11 +358,12 @@ class EnvironmentSetup(Component):
         """Insert default data into the database."""
         db = self.env.get_db_cnx()
         cursor = db.cursor()
-        for table, cols, vals in db_default.data:
+        for table, cols, vals in db_default.get_data(db):
             cursor.executemany("INSERT INTO %s (%s) VALUES (%s)" % (table,
                                ','.join(cols), ','.join(['%s' for c in cols])),
                                vals)
         db.commit()
+        self._update_sample_config()
 
     def environment_needs_upgrade(self, db):
         dbver = self.env.get_version(db)
@@ -354,6 +389,31 @@ class EnvironmentSetup(Component):
                        "name='database_version'", (db_default.db_version,))
         self.log.info('Upgraded database version from %d to %d',
                       dbver, db_default.db_version)
+        self._update_sample_config()
+
+    # Internal methods
+
+    def _update_sample_config(self):
+        from ConfigParser import ConfigParser
+        config = ConfigParser()
+        for section, options in self.config.defaults().items():
+            config.add_section(section)
+            for name, value in options.items():
+                config.set(section, name, value)
+        filename = os.path.join(self.env.path, 'conf', 'trac.ini.sample')
+        try:
+            fileobj = file(filename, 'w')
+            try:
+                config.write(fileobj)
+                fileobj.close()
+            finally:
+                fileobj.close()
+            self.log.info('Wrote sample configuration file with the new '
+                          'settings and their default values: %s',
+                          filename)
+        except IOError, e:
+            self.log.warn('Couldn\'t write sample configuration file (%s)', e,
+                          exc_info=True)
 
 
 def open_environment(env_path=None):
