@@ -1,23 +1,23 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005 Edgewall Software
+# Copyright (C) 2005-2006 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 #         Ludvig Strigeus
 
 from trac.core import *
-from trac.mimeview.api import IHTMLPreviewRenderer
-from trac.util import escape, Markup
+from trac.mimeview.api import content_to_unicode, IHTMLPreviewRenderer, Mimeview
+from trac.util.html import escape, Markup
 from trac.web.chrome import add_stylesheet
 
 __all__ = ['PatchRenderer']
@@ -65,9 +65,9 @@ class PatchRenderer(Component):
     def render(self, req, mimetype, content, filename=None, rev=None):
         from trac.web.clearsilver import HDFWrapper
 
-        tabwidth = int(self.config.get('diff', 'tab_width',
-                       self.config.get('mimeviewer', 'tab_width')))
-        d = self._diff_to_hdf(content.splitlines(), tabwidth)
+        content = content_to_unicode(self.env, content, mimetype)
+        d = self._diff_to_hdf(content.splitlines(),
+                              Mimeview(self.env).tab_width)
         if not d:
             raise TracError, 'Invalid unified diff content'
         hdf = HDFWrapper(loadpaths=[self.env.get_templates_dir(),
@@ -95,11 +95,13 @@ class PatchRenderer(Component):
             for i in xrange(len(fromlines)):
                 fr, to = fromlines[i], tolines[i]
                 (start, end) = _get_change_extent(fr, to)
-                if start != 0 and end != 0:
-                    fromlines[i] = fr[:start] + '\0' + fr[start:end+len(fr)] + \
-                                   '\1' + fr[end:]
-                    tolines[i] = to[:start] + '\0' + to[start:end+len(to)] + \
-                                 '\1' + to[end:]
+                if start != 0 or end != 0:
+                    last = end+len(fr)
+                    fromlines[i] = fr[:start] + '\0' + fr[start:last] + \
+                                   '\1' + fr[last:]
+                    last = end+len(to)
+                    tolines[i] = to[:start] + '\0' + to[start:last] + \
+                                 '\1' + to[last:]
 
         import re
         space_re = re.compile(' ( +)|^ ')
@@ -109,15 +111,24 @@ class PatchRenderer(Component):
 
         output = []
         filename, groups = None, None
-        for line in difflines:
-            if line.startswith('--- '):
+        lines = iter(difflines)
+        try:
+            line = lines.next()
+            while True:
+                if not line.startswith('--- '):
+                    line = lines.next()
+                    continue
+
                 # Base filename/version
                 words = line.split(None, 2)
                 filename, fromrev = words[1], 'old'
                 groups, blocks = None, None
-                continue
-            if line.startswith('+++ '):
+
                 # Changed filename/version
+                line = lines.next()
+                if not line.startswith('+++ '):
+                    return None
+
                 words = line.split(None, 2)
                 if len(words[1]) < len(filename):
                     # Always use the shortest filename for display
@@ -125,49 +136,51 @@ class PatchRenderer(Component):
                 groups = []
                 output.append({'filename' : filename, 'oldrev' : fromrev,
                                'newrev' : 'new', 'diff' : groups})
-                continue
-            # Lines to ignore
-            if line.startswith('Index: ') or line.startswith('======') or line == '':
-                continue
-            if groups == None:
-                return None
-            # @@ -333,10 +329,8 @@
-            if line.startswith('@@ '):
-                r = re.match(r'@@ -(\d+),\d+ \+(\d+),\d+ @@', line)
-                if not r:
-                    return None
-                blocks = []
-                groups.append(blocks)
-                fromline,toline = map(int, r.groups())
-                last_type = None
-                continue
-            if blocks == None:
-                return None
 
-            # First character is the command
-            command, line = line[0], line[1:]
+                for line in lines:
+                    # @@ -333,10 +329,8 @@
+                    r = re.match(r'@@ -(\d+),(\d+) \+(\d+),(\d+) @@', line)
+                    if not r:
+                        break
+                    blocks = []
+                    groups.append(blocks)
+                    fromline,fromend,toline,toend = map(int, r.groups())
+                    last_type = None
 
-            # Make a new block?
-            if (command == ' ') != last_type:
-                last_type = command == ' '
-                blocks.append({'type': last_type and 'unmod' or 'mod',
-                               'base.offset': fromline - 1,
-                               'base.lines': [],
-                               'changed.offset': toline - 1,
-                               'changed.lines': []})
-            if command == ' ':
-                blocks[-1]['changed.lines'].append(line)
-                blocks[-1]['base.lines'].append(line)
-                fromline += 1
-                toline += 1
-            elif command == '+':
-                blocks[-1]['changed.lines'].append(line)
-                toline += 1
-            elif command == '-':
-                blocks[-1]['base.lines'].append(line)
-                fromline += 1
-            else:
-                return None
+                    fromend += fromline
+                    toend += toline
+
+                    while fromline < fromend or toline < toend:
+                        line = lines.next()
+
+                        # First character is the command
+                        command = ' '
+                        if line:
+                            command, line = line[0], line[1:]
+                        # Make a new block?
+                        if (command == ' ') != last_type:
+                            last_type = command == ' '
+                            blocks.append({'type': last_type and 'unmod' or 'mod',
+                                           'base.offset': fromline - 1,
+                                           'base.lines': [],
+                                           'changed.offset': toline - 1,
+                                           'changed.lines': []})
+                        if command == ' ':
+                            blocks[-1]['changed.lines'].append(line)
+                            blocks[-1]['base.lines'].append(line)
+                            fromline += 1
+                            toline += 1
+                        elif command == '+':
+                            blocks[-1]['changed.lines'].append(line)
+                            toline += 1
+                        elif command == '-':
+                            blocks[-1]['base.lines'].append(line)
+                            fromline += 1
+                        else:
+                            return None
+                line = lines.next()
+        except StopIteration:
+            pass
 
         # Go through all groups/blocks and mark up intraline changes, and
         # convert to html
