@@ -541,17 +541,17 @@ class TicketModule(TicketModuleBase):
         internal_cnum = cnum
         if cnum and replyto: # record parent.child relationship
             internal_cnum = '%s.%s' % (replyto, cnum)
-        ticket.save_changes(get_reporter_id(req, 'author'),
-                            req.args.get('comment'), when=now, db=db,
-                            cnum=internal_cnum)
-        db.commit()
+        if ticket.save_changes(get_reporter_id(req, 'author'),
+                               req.args.get('comment'), when=now, db=db,
+                               cnum=internal_cnum):
+            db.commit()
 
-        try:
-            tn = TicketNotifyEmail(self.env)
-            tn.notify(ticket, newticket=False, modtime=now)
-        except Exception, e:
-            self.log.exception("Failure sending notification on change to "
-                               "ticket #%s: %s" % (ticket.id, e))
+            try:
+                tn = TicketNotifyEmail(self.env)
+                tn.notify(ticket, newticket=False, modtime=now)
+            except Exception, e:
+                self.log.exception("Failure sending notification on change to "
+                                   "ticket #%s: %s" % (ticket.id, e))
 
         fragment = cnum and '#comment:'+cnum or ''
         req.redirect(req.href.ticket(ticket.id) + fragment)
@@ -573,6 +573,10 @@ class TicketModule(TicketModuleBase):
             if field['type'] in ('radio', 'select'):
                 value = ticket.values.get(field['name'])
                 options = field['options']
+                if field['name'] == 'milestone' \
+                    and not req.perm.has_permission('TICKET_ADMIN'):
+                    options = [opt for opt in options if not
+                               Milestone(self.env, opt, db=db).is_completed]
                 if value and not value in options:
                     # Current ticket value must be visible even if its not in the
                     # possible values
