@@ -1,23 +1,22 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005 Edgewall Software
+# Copyright (C) 2005-2006 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import generators
-
-from trac.util import TracError
-from trac.versioncontrol import Changeset, Node, Repository, Authorizer
+from trac.core import TracError
+from trac.versioncontrol import Changeset, Node, Repository, Authorizer, \
+                                NoSuchChangeset
 
 
 _kindmap = {'D': Node.DIRECTORY, 'F': Node.FILE}
@@ -32,34 +31,39 @@ class CachedRepository(Repository):
         Repository.__init__(self, repos.name, authz, log)
         self.db = db
         self.repos = repos
-        self.synced = 0
+        self.sync()
 
     def close(self):
         self.repos.close()
 
     def get_changeset(self, rev):
-        if not self.synced:
-            self.sync()
-            self.synced = 1
         return CachedChangeset(self.repos.normalize_rev(rev), self.db,
                                self.authz)
 
+    def get_changesets(self, start, stop):
+        cursor = self.db.cursor()
+        cursor.execute("SELECT rev FROM revision "
+                       "WHERE time >= %s AND time < %s "
+                       "ORDER BY time", (start, stop))
+        for rev, in cursor:
+            if self.authz.has_permission_for_changeset(rev):
+                yield self.get_changeset(rev)
+
     def sync(self):
-        self.log.debug("Checking whether sync with repository is needed")
         cursor = self.db.cursor()
 
         # -- repository used for populating the cache
         cursor.execute("SELECT value FROM system WHERE name='repository_dir'")
-        row = cursor.fetchone()
-        if row:
-            previous_repository_dir = row[0]
+        for previous_repository_dir, in cursor:
+            if previous_repository_dir != self.name:
+                raise TracError("The 'repository_dir' has changed, "
+                                "a 'trac-admin resync' operation is needed.")
+            break
         else: # no 'repository_dir' stored yet, assume everything's OK
-            previous_repository_dir = self.name
-
-        if self.name != previous_repository_dir:
-            raise TracError, ("The 'repository_dir' has changed, "
-                              "a 'trac-admin resync' operation is needed.")
+            cursor.execute("INSERT INTO system (name,value) "
+                           "VALUES ('repository_dir',%s)", (self.name,))
 
+        self.repos.clear()
         youngest_stored = self.repos.get_youngest_rev_in_cache(self.db)
 
         if youngest_stored != str(self.repos.youngest_rev):
@@ -90,8 +94,8 @@ class CachedRepository(Repository):
                                       base_path, base_rev)))
                     kind = kindmap[kind]
                     action = actionmap[action]
-                    cursor.execute("INSERT INTO node_change (rev,path,kind,"
-                                   "change,base_path,base_rev) "
+                    cursor.execute("INSERT INTO node_change (rev,path,"
+                                   "node_type,change_type,base_path,base_rev) "
                                    "VALUES (%s,%s,%s,%s,%s,%s)",
                                    (str(current_rev), path, kind, action,
                                    base_path, base_rev))
@@ -109,13 +113,13 @@ class CachedRepository(Repository):
         return self.repos.oldest_rev
 
     def get_youngest_rev(self):
-        return self.repos.youngest_rev
+        return self.repos.get_youngest_rev_in_cache(self.db)
 
     def previous_rev(self, rev):
         return self.repos.previous_rev(rev)
 
-    def next_rev(self, rev):
-        return self.repos.next_rev(rev)
+    def next_rev(self, rev, path=''):
+        return self.repos.next_rev(rev, path)
 
     def rev_older_than(self, rev1, rev2):
         return self.repos.rev_older_than(rev1, rev2)
@@ -129,6 +133,9 @@ class CachedRepository(Repository):
     def normalize_rev(self, rev):
         return self.repos.normalize_rev(rev)
 
+    def get_changes(self, old_path, old_rev, new_path, new_rev, ignore_ancestry=1):
+        return self.repos.get_changes(old_path, old_rev, new_path, new_rev, ignore_ancestry)
+
 
 class CachedChangeset(Changeset):
 
@@ -143,11 +150,11 @@ class CachedChangeset(Changeset):
             date, author, message = row
             Changeset.__init__(self, rev, message, author, int(date))
         else:
-            raise TracError, "No changeset %s in the repository" % rev
+            raise NoSuchChangeset(rev)
 
     def get_changes(self):
         cursor = self.db.cursor()
-        cursor.execute("SELECT path,kind,change,base_path,base_rev "
+        cursor.execute("SELECT path,node_type,change_type,base_path,base_rev "
                        "FROM node_change WHERE rev=%s "
                        "ORDER BY path", (self.rev,))
         for path, kind, change, base_path, base_rev in cursor:
@@ -157,3 +164,6 @@ class CachedChangeset(Changeset):
             kind = _kindmap[kind]
             change = _actionmap[change]
             yield path, kind, change, base_path, base_rev
+
+    def get_properties(self):
+        return []
