@@ -256,11 +256,13 @@ class SubversionConnector(Component):
         The repository is generally wrapped in a `CachedRepository`,
         unless `direct-svn-fs` is the specified type.
         """
-        authz = None
+        repos = SubversionRepository(dir, None, self.log)
+        crepos = CachedRepository(self.env.get_db_cnx(), repos, None, self.log)
         if authname:
-            authz = SubversionAuthorizer(self.env, authname)
-        repos = SubversionRepository(dir, authz, self.log)
-        return CachedRepository(self.env.get_db_cnx(), repos, authz, self.log)
+            authz = SubversionAuthorizer(self.env, crepos, authname)
+            repos.authz = crepos.authz = authz
+        return crepos
+            
 
 
 class SubversionRepository(Repository):
@@ -302,9 +304,9 @@ class SubversionRepository(Repository):
         else:
             self.scope = '/'
         assert self.scope[0] == '/'
-        
-        self.log.debug("Opening subversion file-system at %s with scope %s" \
-                       % (self.path, self.scope))
+        self.clear()
+
+    def clear(self):
         self.youngest = None
         self.oldest = None
 
@@ -333,7 +335,6 @@ class SubversionRepository(Repository):
         return rev
 
     def close(self):
-        self.log.debug("Closing subversion file-system at %s" % self.path)
         self.repos = None
         self.fs_ptr = None
         self.pool = None
