@@ -1,22 +1,21 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2003-2005 Edgewall Software
-# Copyright (C) 2003-2005 Jonas Borgstrm <jonas@edgewall.com>
+# Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
-# Author: Jonas Borgstrm <jonas@edgewall.com>
+# Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import generators
 try:
     import threading
 except ImportError:
@@ -25,8 +24,9 @@ import time
 import urllib
 import re
 
+from trac.config import BoolOption
 from trac.core import *
-from trac.util import to_utf8
+from trac.util.html import html
 
 
 class IWikiChangeListener(Interface):
@@ -43,6 +43,29 @@ class IWikiChangeListener(Interface):
     def wiki_page_deleted(page):
         """Called when a page has been deleted."""
 
+    def wiki_page_version_deleted(page):
+        """Called when a version of a page has been deleted."""
+
+
+class IWikiPageManipulator(Interface):
+    """Extension point interface for components that need to to specific
+    pre and post processing of wiki page changes.
+    
+    Unlike change listeners, a manipulator can reject changes being committed
+    to the database.
+    """
+
+    def prepare_wiki_page(req, page, fields):
+        """Not currently called, but should be provided for future
+        compatibility."""
+
+    def validate_wiki_page(req, page):
+        """Validate a wiki page after it's been populated from user input.
+        
+        Must return a list of `(field, message)` tuples, one for each problem
+        detected. `field` can be `None` to indicate an overall problem with the
+        page. Therefore, a return value of `[]` means everything is OK."""
+
 
 class IWikiMacroProvider(Interface):
     """Extension point interface for components that provide Wiki macros."""
@@ -61,10 +84,22 @@ class IWikiMacroProvider(Interface):
 class IWikiSyntaxProvider(Interface):
  
     def get_wiki_syntax():
-        """Return an iterable that provides additional wiki syntax."""
+        """Return an iterable that provides additional wiki syntax.
+
+        Additional wiki syntax correspond to a pair of (regexp, cb),
+        the `regexp` for the additional syntax and the callback `cb`
+        which will be called if there's a match.
+        That function is of the form cb(formatter, ns, match).
+        """
  
     def get_link_resolvers():
-        """Return an iterable over (namespace, formatter) tuples."""
+        """Return an iterable over (namespace, formatter) tuples.
+
+        Each formatter should be a function of the form
+        fmt(formatter, ns, target, label), and should
+        return some HTML fragment.
+        The `label` is already HTML escaped, whereas the `target` is not.
+        """
  
 
 class WikiSystem(Component):
@@ -78,6 +113,14 @@ class WikiSystem(Component):
 
     INDEX_UPDATE_INTERVAL = 5 # seconds
 
+    ignore_missing_pages = BoolOption('wiki', 'ignore_missing_pages', 'false',
+        """Enable/disable highlighting CamelCase links to missing pages
+        (''since 0.9'').""")
+
+    split_page_names = BoolOption('wiki', 'split_page_names', 'false',
+        """Enable/disable splitting the WikiPageNames with space characters
+        (''since 0.10'').""")
+
     def __init__(self):
         self._index = None
         self._last_index_update = 0
@@ -119,7 +162,7 @@ class WikiSystem(Component):
     def has_page(self, pagename):
         """Whether a page with the specified name exists."""
         self._update_index()
-        return self._index.has_key(pagename)
+        return self._index.has_key(pagename.rstrip('/'))
 
     def _get_rules(self):
         self._prepare_rules()
@@ -135,7 +178,7 @@ class WikiSystem(Component):
         self._prepare_rules()
         return self._external_handlers
     external_handlers = property(_get_external_handlers)
-    
+
     def _prepare_rules(self):
         from trac.wiki.formatter import Formatter
         if not self._compiled_rules:
@@ -145,7 +188,7 @@ class WikiSystem(Component):
             i = 0
             for resolver in self.syntax_providers:
                 for regexp, handler in resolver.get_wiki_syntax():
-                    handlers['i'+str(i)] = handler
+                    handlers['i' + str(i)] = handler
                     syntax.append('(?P<i%d>%s)' % (i, regexp))
                     i += 1
             syntax += Formatter._post_rules[:]
@@ -182,34 +225,63 @@ class WikiSystem(Component):
             self.log.debug('Removing page %s from index' % page.name)
             del self._index[page.name]
 
+    def wiki_page_version_deleted(self, page):
+        pass
+
     # IWikiSyntaxProvider methods
+
+    XML_NAME = r"[\w:](?<!\d)(?:[\w:.-]*[\w-])?"
+    # See http://www.w3.org/TR/REC-xml/#id,
+    # here adapted to exclude terminal "." and ":" characters
+
+    def format_page_name(self, page):
+        if self.split_page_names:
+            return re.sub(r"([a-z])([A-Z][a-z])", r"\1 \2", page)
+        return page
     
     def get_wiki_syntax(self):
-        ignore_missing = self.config.getbool('wiki', 'ignore_missing_pages')
-        yield (r"!?(?<!/)\b[A-Z][a-z]+(?:[A-Z][a-z]*[a-z/])+"
-                "(?:#[A-Za-z0-9]+)?(?=\Z|\s|[.,;:!?\)}\]])",
-               lambda x, y, z: self._format_link(x, 'wiki', y, y,
-                                                 ignore_missing))
+        from trac.wiki.formatter import Formatter
+        wiki_page_name = (
+            r"[A-Z][a-z]+(?:[A-Z][a-z]*[a-z/])+" # wiki words
+            r"(?:#%s)?" % self.XML_NAME + # optional fragment id
+            r"(?=:(?:\Z|\s)|[^:a-zA-Z]|\s|\Z)" # what should follow it
+            )
+        
+        # Regular WikiPageNames
+        def wikipagename_link(formatter, match, fullmatch):
+            return self._format_link(formatter, 'wiki', match,
+                                     self.format_page_name(match),
+                                     self.ignore_missing_pages)
+        
+        yield (r"!?(?<!/)\b" + # start at a word boundary but not after '/'
+               wiki_page_name, wikipagename_link)
+
+        # [WikiPageNames with label]
+        def wikipagename_with_label_link(formatter, match, fullmatch):
+            page, label = match[1:-1].split(' ', 1)
+            return self._format_link(formatter, 'wiki', page, label.strip(),
+                                     self.ignore_missing_pages)
+        yield (r"!?\[%s\s+(?:%s|[^\]]+)\]" % (wiki_page_name,
+                                              Formatter.QUOTED_STRING),
+               wikipagename_with_label_link)
+
+        # MoinMoin's ["internal free link"] 
+        def internal_free_link(fmt, m, fullmatch): 
+            return self._format_link(fmt, 'wiki', m[2:-2], m[2:-2], False) 
+        yield (r"!?\[(?:%s)\]" % Formatter.QUOTED_STRING, internal_free_link) 
 
     def get_link_resolvers(self):
-        yield ('wiki', self._format_fancy_link)
-
-    def _format_fancy_link(self, f, n, p, l):
-        return self._format_link(f, n, p, l, False)
+        def link_resolver(formatter, ns, target, label):
+            return self._format_link(formatter, ns, target, label, False)
+        yield ('wiki', link_resolver)
 
     def _format_link(self, formatter, ns, page, label, ignore_missing):
-        anchor = ''
-        if page.find('#') != -1:
-            anchor = page[page.find('#'):]
-            page = page[:page.find('#')]
-        page = urllib.unquote(page)
-        label = urllib.unquote(label)
-
+        page, query, fragment = formatter.split_link(page)
+        href = formatter.href.wiki(page) + fragment
         if not self.has_page(page):
             if ignore_missing:
                 return label
-            return '<a class="missing wiki" href="%s" rel="nofollow">%s?</a>' \
-                   % (formatter.href.wiki(page) + anchor, label)
+            return html.A(label+'?', href=href, class_='missing wiki',
+                          rel='nofollow')
         else:
-            return '<a class="wiki" href="%s">%s</a>' \
-                   % (formatter.href.wiki(page) + anchor, label)
+            return html.A(label, href=href, class_='wiki')
